@@ -1,7 +1,7 @@
-package SimpleFileSelect;
-my $RCSRevKey = '$Revision: 0.60 $';
+package Tk::SimpleFileSelect;
+my $RCSRevKey = '$Revision: 0.65 $';
 $RCSRevKey =~ /Revision: (.*?) /;
-$VERSION=$1;
+$VERSION=0.63;
 use vars qw($VERSION @EXPORT_OK);
 @EXPORT_OK = qw(glob_to_re);
 
@@ -14,19 +14,16 @@ use File::Basename;
 
 Construct Tk::Widget 'SimpleFileSelect';
 
-sub Cancel
-{
+sub Cancel {
   my ($cw) = @_;
   $cw -> withdraw;
   return '';
 }
 
-sub Accept_dir
-{
+sub Accept_dir {
  my ($cw,$new) = @_;
  my $dir  = $cw->cget('-directory');
- $cw->configure(-directory => "$dir/$new",
-	       -title => "$dir/$new" );
+ $cw->configure(-directory => "$dir/$new", -title => "$dir/$new" );
 }
 
 sub Open {
@@ -54,12 +51,31 @@ sub Populate {
     require Tk::LabEntry;
     require Cwd;
     $w->SUPER::Populate($args);
+
+    $w->ConfigSpecs(
+        -font             => ['CHILDREN',undef,undef,
+			      '*-helvetica-medium-r-*-*-12-*'],
+        -width            => [ ['dir_list'], undef, undef, 30 ],
+        -height           => [ ['dir_list'], undef, undef, 14 ],
+        -directory        => [ 'METHOD', undef, undef, '.' ],
+        -initialdir       => '-directory',
+        -files            => [ 'PASSIVE', undef,undef,1 ],
+        -selectedfile     => [ 'PASSIVE', undef,undef,''],
+        -dotfiles         => [ 'PASSIVE', undef,undef,0 ],
+        -filter           => [ 'METHOD',  undef, undef, undef ],
+        '-accept'         => [ 'CALLBACK',undef,undef, undef ],
+        -create           => [ 'PASSIVE', undef, undef, 0 ],
+        -acceptlabel      => [ 'PASSIVE', undef, undef, 'Accept' ],
+	-initialtext      => [ 'PASSIVE', undef, undef, '' ],
+        DEFAULT           => [ 'dir_list' ],
+    );
+
     $w->protocol('WM_DELETE_WINDOW' => ['Cancel', $w ]);
     $w->{'reread'} = 0;
     my $l = $w -> Component( Label => 'entry_label',-text => 'File Name: ');
     $l -> grid( -column => 1, -row => 3, -padx => 5, -pady => 5 );
     my $e = $w -> Component(Entry => 'file_entry',
-        -textvariable => \$w->{Configure}{-initialfile} );
+		    -textvariable=>\$w->{Configure}{-initialtext});
     $e->grid(-column => 2, -columnspan => 1, -padx => 5, -pady => 5,
 	     -row => 3, -sticky => 'e,w' );
     $e->bind('<Return>' => [$w => 'Open', Ev(['getSelected'])]);
@@ -71,7 +87,7 @@ sub Populate {
     $lb->grid( -column => 2, -row => 1, -rowspan => 2, -padx => 5,
 	    -pady => 5, -sticky => 'nsew' );
     $lb->bind('<Double-Button-1>' => [$w => 'Open', Ev(['getSelected'])]);
-    $lb->bind('<Button-1>', sub {($w->{Configure}{-initialfile}=
+    $lb->bind('<Button-1>', sub {($w->{Configure}{-selectedfile}=
 	    $lb->get($lb->curselection))&&($e->icursor( 'end' ))});
     $b = $w -> Button(-textvariable => \$w->{'Configure'}{'-acceptlabel'},
     -underline => 0,-command => [$w => 'Open', Ev(['getSelected']) ]);
@@ -84,240 +100,162 @@ sub Populate {
     $w -> Subwidget('file_entry') -> focus;
     $w -> eventAdd( '<<Accept>>', '<Alt-a>');
     $w -> bind('<<Accept>>', [$w => 'Open', Ev(['getSelected']) ]);
-    $w->ConfigSpecs(
-        -font             => ['CHILDREN',undef,undef,
-			      '*-helvetica-medium-r-*-*-12-*'],
-        -width            => [ ['dir_list'], undef, undef, 30 ],
-        -height           => [ ['dir_list'], undef, undef, 14 ],
-        -directory        => [ 'METHOD', undef, undef, '.' ],
-        -initialdir       => '-directory',
-        -files            => [ 'PASSIVE', undef,undef,1 ],
-        -dotfiles         => [ 'PASSIVE', undef,undef,0 ],
-        -initialfile      => [ 'PASSIVE', undef, undef, '' ],
-        -filter           => [ 'METHOD',  undef, undef, undef ],
-        '-accept'         => [ 'CALLBACK',undef,undef, undef ],
-        -create           => [ 'PASSIVE', undef, undef, 0 ],
-        -acceptlabel      => [ 'PASSIVE', undef, undef, 'Accept' ],
-        DEFAULT           => [ 'dir_list' ],
-    );
     $w->Delegates(DEFAULT => 'dir_list');
     return $w;
 }
 
-sub translate
-  {
-      my ($bs,$ch) = @_;
-      return "\\$ch" if (length $bs);
-      return '.*'  if ($ch eq '*');
- return '.'   if ($ch eq '?');
- return "\\."  if ($ch eq '.');
- return "\\/" if ($ch eq '/');
- return "\\\\" if ($ch eq '\\');
- return $ch;
+sub translate {
+    my ($bs,$ch) = @_;
+    return "\\$ch" if (length $bs);
+    return '.*'  if ($ch eq '*');
+    return '.'   if ($ch eq '?');
+    return "\\."  if ($ch eq '.');
+    return "\\/" if ($ch eq '/');
+    return "\\\\" if ($ch eq '\\');
+    return $ch;
 }
 
-sub glob_to_re
-{
- my $regex = shift;
- $regex =~ s/(\\?)(.)/&translate($1,$2)/ge;
- return sub { shift =~ /^${regex}$/ };
+sub glob_to_re {
+    my $regex = shift;
+    $regex =~ s/(\\?)(.)/&translate($1,$2)/ge;
+    return sub { shift =~ /^${regex}$/ };
 }
 
-sub filter
-{
- my ($cw,$val) = @_;
- my $var = \$cw->{Configure}{'-filter'};
- if (@_ > 1 || !defined($$var))
-  {
-   $val = '*' unless defined $val;
-   $$var = $val;
-   $cw->{'match'} = glob_to_re($val)  unless defined $cw->{'match'};
-   unless ($cw->{'reread'}++)
-    {
-     $cw->Busy;
-     if( ( $cw -> cget( '-connected' ) ) =~ /1/ ) {
-       $cw->afterIdle(['rereadRemote',$cw,$cw->cget('-directory')])
-     } else {
-       $cw->afterIdle(['reread',$cw,$cw->cget('-directory')])
-     }
+sub filter {
+    my ($cw,$val) = @_;
+    my $var = \$cw->{Configure}{'-filter'};
+    if (@_ > 1 || !defined($$var)) {
+	$val = '*' unless defined $val;
+	$$var = $val;
+	$cw->{'match'} = glob_to_re($val)  unless defined $cw->{'match'};
+	unless ($cw->{'reread'}++) {
+	    $cw->Busy;
+	    if( ( $cw -> cget( '-connected' ) ) =~ /1/ ) {
+		$cw->afterIdle(['rereadRemote',$cw,$cw->cget('-directory')])
+		} else {
+		    $cw->afterIdle(['reread',$cw,$cw->cget('-directory')]);
+		}
+	}
     }
-  }
- return $$var;
+    return $$var;
 }
 
-sub defaultextension
-{
- my ($cw,$val) = @_;
- if (@_ > 1)
-  {
-   $val = ".$val" if ($val !~ /^\./);
-   $cw->filter("*$val");
-  }
- else
-  {
-   $val = $cw->filter;
-   my ($ext) = $val =~ /(\.[^\.]*)$/;
-   return $ext;
-  }
-}
-
-sub directory
-{
- my ($cw,$dir) = @_;
- my $var = \$cw->{Configure}{'-directory'};
- if (@_ > 1 && defined $dir)
-  {
-   if (substr($dir,0,1) eq '~')
-    {
-     if (substr($dir,1,1) eq '/')
-      {
-       $dir = $ENV{'HOME'} . substr($dir,1);
-      }
-     else
-      {my ($uid,$rest) = ($dir =~ m#^~([^/]+)(/.*$)#);
-       $dir = (getpwnam($uid))[7] . $rest;
-      }
+sub defaultextension {
+    my ($cw,$val) = @_;
+    if (@_ > 1) {
+	$val = ".$val" if ($val !~ /^\./);
+	$cw->filter("*$val");
+    } else {
+	$val = $cw->filter;
+	my ($ext) = $val =~ /(\.[^\.]*)$/;
+	return $ext;
     }
-   $dir =~ s#([^/\\])[\\/]+$#$1#;
-   if (-d $dir)
-    {
-     unless (Tk::tainting())
-      {
-       my $pwd = Cwd::getcwd();
-       if (chdir( (defined($dir) ? $dir : '') ) )
-        {
-         my $new = Cwd::getcwd();
-         if ($new)
-          {
-           $dir = $new;
-          }
-         else
-          {
-           carp "Cannot getcwd in '$dir'";
-          }
-         chdir($pwd) || carp "Cannot chdir($pwd) : $!";
-         $cw->{Configure}{'-directory'} = $dir;
-	 $cw->configure(-title=>$dir);
-        }
-       else
-        {
-         $cw->BackTrace("Cannot chdir($dir) :$!");
-        }
-      }
-     $$var = $dir;
-     unless ($cw->{'reread'}++)
-      {
-       $cw->Busy;
-       $cw->afterIdle(['reread',$cw])
-      }
+}
+
+sub directory {
+    my ($cw,$dir) = @_;
+    my $var = \$cw->{Configure}{'-directory'};
+    if (@_ > 1 && defined $dir) {
+	if (substr($dir,0,1) eq '~') {
+	    if (substr($dir,1,1) eq '/') {
+		$dir = $ENV{'HOME'} . substr($dir,1);
+	    } else {
+		my ($uid,$rest) = ($dir =~ m#^~([^/]+)(/.*$)#);
+		$dir = (getpwnam($uid))[7] . $rest;
+	    }
+	}
+	$dir =~ s#([^/\\])[\\/]+$#$1#;
+	if (-d $dir) {
+	    unless (Tk::tainting()) {
+		my $pwd = Cwd::getcwd();
+		if (chdir( (defined($dir) ? $dir : '') ) ) {
+		    my $new = Cwd::getcwd();
+		    if ($new) {
+			$dir = $new;
+		    } else {
+			carp "Cannot getcwd in '$dir'";
+		    }
+		    chdir($pwd) || carp "Cannot chdir($pwd) : $!";
+		    $cw->{Configure}{'-directory'} = $dir;
+		    $cw->configure(-title=>$dir);
+		} else {
+		    $cw->BackTrace("Cannot chdir($dir) :$!");
+		}
+	    }
+	    $$var = $dir;
+	    unless ($cw->{'reread'}++) {
+		$cw->Busy;
+		$cw->afterIdle(['reread',$cw]);
+	    }
+	}
     }
-  }
- return $$var;
+    return $$var;
 }
 
 
-sub reread
-{
-  my ($w) = @_;
-  my $dir = $w->cget('-directory');
-  my ($f, $seen);
- if (defined $dir)
-  {
-   if (!defined $w->cget('-filter') or $w->cget('-filter') eq '')
-    {
-     $w->configure('-filter', '*');
+
+sub reread {
+    my ($w) = @_;
+    my $dir = $w->cget('-directory');
+    my ($f, $seen);
+    if (defined $dir) {
+	if (!defined $w->cget('-filter') or $w->cget('-filter') eq '') {
+	    $w->configure('-filter', '*');
+	}
+	my $dl = $w->Subwidget('dir_list');
+	$dl->delete(0, 'end');
+	local *DIR;
+	my $h;
+	if (opendir(DIR, $dir)) {
+	    my $file = $w->cget('-selectedfile');
+	    my $seen = 0;
+	    my $accept = $w->cget('-accept');
+	    foreach $f (sort(readdir(DIR))) {
+		next if ($f eq '.');
+		next if $f =~ /^\.[^\.]/ and ! $w -> {Configure}{-dotfiles} ;
+		my $path = "$dir/$f";
+		if (-d $path) {
+		    $dl->insert('end', $f.'/');
+		} elsif ($w -> cget('-files')) {
+		    if (&{$w->{match}}($f)) {
+			if (!defined($accept) || $accept->Call($path)) {
+			    $seen = $dl->index('end') 
+				if ($file && $f eq $file);
+			    $dl->insert('end', $f);
+			}
+		    }
+		}
+	    }
+	    closedir(DIR);
+	    if ($seen) {
+		$dl->selectionSet($seen);
+		$dl->see($seen);
+	    } else {
+		$w->configure(-selectedfile => '') 
+		    unless $w->cget('-create');
+	    }
+	}
+	$w->{DirectoryString} = $dir . '/' . $w->cget('-filter');
+	$w->{'reread'} = 0;
+	$w->Unbusy;
     }
-   my $dl = $w->Subwidget('dir_list');
-   $dl->delete(0, 'end');
-   local *DIR;
-   my $h;
-   if (opendir(DIR, $dir)) {
-     my $file = $w->cget('-initialfile');
-     my $seen = 0;
-     my $accept = $w->cget('-accept');
-     foreach $f (sort(readdir(DIR))) {
-       next if ($f eq '.');
-       next if $f =~ /^\.[^\.]/ and ! $w -> {Configure}{-dotfiles} ;
-       my $path = "$dir/$f";
-       if (-d $path) {
-	 $dl->insert('end', $f.'/');
-       } elsif( $w -> cget('-files')) {
-	 if (&{$w->{match}}($f)) {
-	   if (!defined($accept) || $accept->Call($path)) {
-	     $seen = $dl->index('end') if ($file && $f eq $file);
-	     $dl->insert('end', $f)
-	   }
-	 }
-       }
-     }
-     closedir(DIR);
-     if ($seen) {
-       $dl->selectionSet($seen);
-       $dl->see($seen);
-     }
-     else {
-       $w->configure(-initialfile => undef) unless $w->cget('-create');
-     }
-   }
-   $w->{DirectoryString} = $dir . '/' . $w->cget('-filter');
-   $w->{'reread'} = 0;
-   $w->Unbusy;
- }
 }
 
-sub validateDir
-{
- my ($cw,$name) = @_;
- if( ( $cw -> cget( '-connected' ) ) =~ /1/ ) {
-   $name =~ s/^.*\://;
- }
- my ($leaf,$base) = fileparse($name);
- if ($leaf =~ /[*?]/)
-  {
-   $cw->configure('-directory' => $base,'-filter' => $leaf);
-  }
- else
-  {
-   $cw->configure('-directory' => $name);
-  }
+sub Error {
+    my $cw  = shift;
+    my $msg = shift;
+    my $dlg = $cw->Subwidget('dialog');
+    $dlg->configure(-text => $msg);
+    $dlg->Show;
 }
 
-sub validateFile
-{
- my ($cw,$name) = @_;
- my $path = $cw -> {'Configure'}{'-directory'};
- $cw -> {Selected} = "$path/$name";
-}
-
-sub Error
-{
- my $cw  = shift;
- my $msg = shift;
- my $dlg = $cw->Subwidget('dialog');
- $dlg->configure(-text => $msg);
- $dlg->Show;
-}
-
-sub Show
-{
- my ($cw,@args) = @_;
- my $accel = lc(substr $cw->cget('-acceptlabel'),0,1);
- $cw -> eventAdd('<<Accept>>', "\<Alt-$accel\>");
- $cw -> waitVariable(\$cw->{Selected});
- $cw -> withdraw;
- return $cw -> {Selected};
-}
-
-sub FDialog
-{
- my($cmd, %args) = @_;
- if ($cmd =~ /Save/)
-  {
-   $args{-create} = 1;
-  }
- delete $args{-filetypes};
- delete $args{-force};
- Tk::DialogWrapper('FileSelect',$cmd, %args);
+sub Show {
+    my ($cw,@args) = @_;
+    my $accel = lc(substr $cw->cget('-acceptlabel'),0,1);
+    $cw -> eventAdd('<<Accept>>', "\<Alt-$accel\>");
+    $cw -> waitVariable(\$cw->{Selected});
+    $cw -> withdraw;
+    return $cw -> {Selected};
 }
 
 1;
@@ -378,17 +316,15 @@ directory selection. Defaults to 'Accept'.  The first character
 is underlined to correspond with an Alt- accelerator constructed
 with the first letter of the label text.
 
-=item -initialfile
-
-Text to display initially in the file entry label, and select,
-if it exists, in the directory Listbox. The default is ''
-(empty string).
-
 =item -filter
 
 Display only files matching this pattern.  The default is
 '*' (all files).
 
+=item -initialtext
+
+The text to appear in the entry box when the widget is opened.
+
 =head1 DESCRIPTION
 
 Tk::SimpleFileSelect is a easy-to-use file selection widget based
@@ -409,9 +345,13 @@ The return value is the pathname of a file selected in the
 Listbox, or the fully qualified path name of a file given in the
 text entry, or an empty string if no file name is specified.
 
+=head1 ADVERTISED SUBWIDGETS
+
+None.
+
 =head1 VERSION INFORMATION
 
-  $Revision: 0.60 $
+  $Id: SimpleFileSelect.pm,v 0.65 2002/03/24 21:34:56 kiesling Exp $
 
 =head1 COPYRIGHT INFO
 
